﻿using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ZooContracts.BindingModels;
using ZooContracts.BusinessLogicsContracts;
using ZooContracts.SearchModel;
using ZooContracts.StoragesContracts;
using ZooContracts.ViewModels;
using ZooDatabaseImplements.Implements;

namespace ZooBusinessLogic.BusinessLogics
{
    public class RouteCostLogic : IRouteCostLogic
    {
        private readonly ILogger _logger;
        private readonly IRouteCostStorage _RouteCostStorage;
        public RouteCostLogic(ILogger<RouteCostLogic> logger, IRouteCostStorage RouteCostStorage)
        {
            _logger = logger;
            _RouteCostStorage = RouteCostStorage;
        }

        public bool Create(RouteCostBindingModel model)
        {
            CheckModel(model);
            if (_RouteCostStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }

        public bool Delete(RouteCostBindingModel model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id: {Id}", model.Id);
            if (_RouteCostStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }

        public RouteCostViewModel? ReadElement(RouteCostSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. Id: {Id}", model.Id);
            var element = _RouteCostStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement found. Id: {Id}", element.Id);
            return element;
        }

        public List<RouteCostViewModel>? ReadList(RouteCostSearchModel? model)
        {
            _logger.LogInformation("ReadList. Id: {Id}", model?.Id);
            var list = model == null ? _RouteCostStorage.GetFullList() : _RouteCostStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count: {Count}", list.Count);
            return list;
        }

        public bool Update(RouteCostBindingModel model)
        {
            CheckModel(model);
            if (_RouteCostStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }

        private void CheckModel(RouteCostBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            _logger.LogInformation("RouteCost. Id: {Id}", model.Id);
        }
    }
}
