﻿using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ZooContracts.BusinessLogicsContracts;
using ZooContracts.BusinessLogicsContracts;
using ZooContracts.SearchModel;
using ZooContracts.StoragesContracts;
using ZooContracts.ViewModels;
using ZooDatabaseImplements.Implements;


namespace ZooBusinessLogic.BusinessLogics
{
    public class RouteLogic : IRouteLogic
    {
        private readonly ILogger _logger;
        private readonly IRouteStorage _RouteStorage;
        public RouteLogic(ILogger<RouteLogic> logger, IRouteStorage RouteStorage)
        {
            _logger = logger;
            _RouteStorage = RouteStorage;
        }

        public bool Create(RouteBindingModel model)
        {
            CheckModel(model);
            if (_RouteStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }

        public bool Delete(RouteBindingModel model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id: {Id}", model.Id);
            if (_RouteStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }

        public RouteViewModel? ReadElement(RouteSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. Id: {Id}", model.Id);
            var element = _RouteStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement found. Id: {Id}", element.Id);
            return element;
        }

        public List<RouteViewModel>? ReadList(RouteSearchModel? model)
        {
            _logger.LogInformation("ReadList. Id: {Id}", model?.Id);
            var list = model == null ? _RouteStorage.GetFullList() : _RouteStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count: {Count}", list.Count);
            return list;
        }

        public bool Update(RouteBindingModel model)
        {
            CheckModel(model);
            if (_RouteStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }
        private void CheckModel(RouteBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            _logger.LogInformation("Route. Id: {Id}", model.Id);
        }
    }
}
